@@ -8,8 +8,13 @@ from celery import signals
 from celery.loaders.base import BaseLoader
 from celery.datastructures import DictAttribute
 
+from django import db
+from django.conf import settings
+from django.core import cache
 from django.core.mail import mail_admins
 
+from .utils import DATABASE_ERRORS, now
+
 _RACE_PROTECTION = False
 
 
@@ -30,9 +35,11 @@ class DjangoLoader(BaseLoader):
         # any embedded celerybeat process forks.
         signals.beat_embedded_init.connect(self.close_database)
 
+    def now(self):
+        return now()
+
     def read_configuration(self):
         """Load configuration from Django settings."""
-        from django.conf import settings
         self.configured = True
         # Default backend needs to be the database backend for backward
         # compatibility.
@@ -42,19 +49,30 @@ class DjangoLoader(BaseLoader):
             settings.CELERY_RESULT_BACKEND = "database"
         return DictAttribute(settings)
 
+    def _close_database(self):
+        try:
+            funs = [conn.close for conn in db.connections]
+        except AttributeError:
+            funs = [db.close_connection]  # pre multidb
+
+        for close in funs:
+            try:
+                close()
+            except DATABASE_ERRORS, exc:
+                if "closed" not in str(exc):
+                    raise
+
     def close_database(self, **kwargs):
-        import django.db
         db_reuse_max = getattr(self.conf, "CELERY_DB_REUSE_MAX", None)
         if not db_reuse_max:
-            return django.db.close_connection()
+            return self._close_database()
         if self._db_reuse >= db_reuse_max * 2:
             self._db_reuse = 0
-            return django.db.close_connection()
+            self._close_database()
         self._db_reuse += 1
 
     def close_cache(self):
         try:
-            from django.core import cache
             cache.cache.close()
         except (TypeError, AttributeError):
             pass
@@ -81,7 +99,6 @@ class DjangoLoader(BaseLoader):
 
         """
 
-        from django.conf import settings
         if settings.DEBUG:
             warnings.warn("Using settings.DEBUG leads to a memory leak, never "
                           "use this setting in production environments!")
@@ -89,7 +106,13 @@ class DjangoLoader(BaseLoader):
         self.close_database()
         self.close_cache()
         self.import_default_modules()
-        autodiscover()
+
+    def import_default_modules(self):
+        super(DjangoLoader, self).import_default_modules()
+        self.autodiscover()
+
+    def autodiscover(self):
+        self.task_modules.update(mod.__name__ for mod in autodiscover() or ())
 
     def on_worker_process_init(self):
         # the parent process may have established these,
@@ -103,7 +126,6 @@ class DjangoLoader(BaseLoader):
 
 def autodiscover():
     """Include tasks for all applications in ``INSTALLED_APPS``."""
-    from django.conf import settings
     global _RACE_PROTECTION
 
     if _RACE_PROTECTION:
