@@ -2,11 +2,11 @@ from __future__ import absolute_import
 
 import os
 import sys
+import types
 
 DEFAULT_APPS = ("django.contrib.auth",
                 "django.contrib.contenttypes",
                 "django.contrib.sessions",
-                "django.contrib.sites",
                 "django.contrib.admin",
                 "django.contrib.admindocs",
                 "djcelery")
@@ -14,22 +14,33 @@ DEFAULT_APPS = ("django.contrib.auth",
 DEFAULTS = {"ROOT_URLCONF": "djcelery.monproj.urls",
             "DATABASE_ENGINE": "sqlite3",
             "DATABASE_NAME": "djcelerymon.db",
-            "BROKER_HOST": "localhost",
-            "BROKER_USER": "guest",
-            "BROKER_PASSWORD": "guest",
-            "BROKER_VHOST": "/",
+            "DATABASES": {"default": {
+                            "ENGINE": "django.db.backends.sqlite3",
+                            "NAME": "djcelerymon.db"}},
+            "BROKER_URL": "amqp://",
             "SITE_ID": 1,
             "INSTALLED_APPS": DEFAULT_APPS,
-            "DEBUG": True}
+            "DEBUG": False}
+
+
+def default_settings(name="__default_settings__"):
+    c = type(name, (types.ModuleType, ), DEFAULTS)(name)
+    c.__dict__.update({"__file__": __file__})
+    sys.modules[name] = c
+    return name
 
 
 def configure():
     from celery import current_app
+    from celery.loaders.default import DEFAULT_CONFIG_MODULE
     from django.conf import settings
 
     if not settings.configured:
-        settings_module = os.environ.get("CELERY_CONFIG_MODULE",
-                                         "celeryconfig")
+        if current_app.loader.configured:
+            settings_module = os.environ.get("CELERY_CONFIG_MODULE",
+                                             DEFAULT_CONFIG_MODULE)
+        else:
+            settings_module = default_settings()
         settings.configure(SETTINGS_MODULE=settings_module,
                            **dict(DEFAULTS, **current_app.loader.conf))
         settings.DEBUG = True
