@@ -6,7 +6,7 @@ import os
 # If your extensions are in another directory, add it here. If the directory
 # is relative to the documentation root, use os.path.abspath to make it
 # absolute, like shown here.
-sys.path.append(os.path.join(os.pardir, "tests"))
+sys.path.insert(0, os.getcwd())
 from django.core.management import setup_environ
 import settings
 setup_environ(settings)
@@ -75,9 +75,6 @@ html_sidebars = {
 }
 
 ### Issuetracker
-
-# this is not working for some reason...
-if False:
-    issuetracker = "github"
-    issuetracker_project = "ask/django-celery"
-    issuetracker_issue_pattern = r'[Ii]ssue #(\d+)'
+issuetracker = "github"
+issuetracker_project = "ask/django-celery"
+issuetracker_issue_pattern = r'[Ii]ssue #(\d+)'
